pub struct WorkTable<Row, PrimaryKey, AvailableTypes = (), AvailableIndexes = (), SecondaryIndexes = (), LockType = (), PkGen = <PrimaryKey as TablePrimaryKey>::Generator, const DATA_LENGTH: usize = INNER_PAGE_SIZE, PkNodeType = Vec<Pair<PrimaryKey, OffsetEqLink<DATA_LENGTH>>>>where
PrimaryKey: Clone + Ord + Send + 'static + Hash,
Row: StorableRow + Send + Clone + 'static,
PkNodeType: NodeLike<Pair<PrimaryKey, OffsetEqLink<DATA_LENGTH>>> + Send + 'static,{
pub data: Arc<DataPages<Row, DATA_LENGTH>>,
pub primary_index: Arc<PrimaryIndex<PrimaryKey, DATA_LENGTH, PkNodeType>>,
pub indexes: Arc<SecondaryIndexes>,
pub pk_gen: PkGen,
pub lock_manager: Arc<LockMap<LockType, PrimaryKey>>,
pub update_state: IndexMap<PrimaryKey, Row>,
pub table_name: &'static str,
pub pk_phantom: PhantomData<(AvailableTypes, AvailableIndexes)>,
}Fields
data: Arc<DataPages<Row, DATA_LENGTH>>primary_index: Arc<PrimaryIndex<PrimaryKey, DATA_LENGTH, PkNodeType>>indexes: Arc<SecondaryIndexes>pk_gen: PkGenlock_manager: Arc<LockMap<LockType, PrimaryKey>>update_state: IndexMap<PrimaryKey, Row>table_name: &'static strpk_phantom: PhantomData<(AvailableTypes, AvailableIndexes)>Implementations
Sourceimpl<Row, PrimaryKey, AvailableTypes, AvailableIndexes, SecondaryIndexes, LockType, PkGen, const DATA_LENGTH: usize, NodeType> WorkTable<Row, PrimaryKey, AvailableTypes, AvailableIndexes, SecondaryIndexes, LockType, PkGen, DATA_LENGTH, NodeType>where
PrimaryKey: Debug + Clone + Ord + Send + 'static + Hash,
Row: StorableRow + Send + Clone + 'static,
<Row as StorableRow>::WrappedRow: RowWrapper<Row>,
NodeType: NodeLike<Pair<PrimaryKey, OffsetEqLink<DATA_LENGTH>>> + Send + 'static,
SecondaryIndexes: MemStat + TableSecondaryIndexInfo,
impl<Row, PrimaryKey, AvailableTypes, AvailableIndexes, SecondaryIndexes, LockType, PkGen, const DATA_LENGTH: usize, NodeType> WorkTable<Row, PrimaryKey, AvailableTypes, AvailableIndexes, SecondaryIndexes, LockType, PkGen, DATA_LENGTH, NodeType>where
PrimaryKey: Debug + Clone + Ord + Send + 'static + Hash,
Row: StorableRow + Send + Clone + 'static,
<Row as StorableRow>::WrappedRow: RowWrapper<Row>,
NodeType: NodeLike<Pair<PrimaryKey, OffsetEqLink<DATA_LENGTH>>> + Send + 'static,
SecondaryIndexes: MemStat + TableSecondaryIndexInfo,
pub fn system_info(&self) -> SystemInfo
Sourceimpl<Row, PrimaryKey, AvailableTypes, AvailableIndexes, SecondaryIndexes, LockType, PkGen, const DATA_LENGTH: usize, PkNodeType> WorkTable<Row, PrimaryKey, AvailableTypes, AvailableIndexes, SecondaryIndexes, LockType, PkGen, DATA_LENGTH, PkNodeType>where
Row: TableRow<PrimaryKey> + StorableRow + Send + Clone + 'static,
PrimaryKey: Debug + Clone + Ord + Send + TablePrimaryKey + Hash,
PkNodeType: NodeLike<Pair<PrimaryKey, OffsetEqLink<DATA_LENGTH>>> + Send + 'static,
<Row as StorableRow>::WrappedRow: RowWrapper<Row>,
impl<Row, PrimaryKey, AvailableTypes, AvailableIndexes, SecondaryIndexes, LockType, PkGen, const DATA_LENGTH: usize, PkNodeType> WorkTable<Row, PrimaryKey, AvailableTypes, AvailableIndexes, SecondaryIndexes, LockType, PkGen, DATA_LENGTH, PkNodeType>where
Row: TableRow<PrimaryKey> + StorableRow + Send + Clone + 'static,
PrimaryKey: Debug + Clone + Ord + Send + TablePrimaryKey + Hash,
PkNodeType: NodeLike<Pair<PrimaryKey, OffsetEqLink<DATA_LENGTH>>> + Send + 'static,
<Row as StorableRow>::WrappedRow: RowWrapper<Row>,
pub fn get_next_pk(&self) -> PrimaryKeywhere
PkGen: PrimaryKeyGenerator<PrimaryKey>,
Sourcepub fn select(&self, pk: PrimaryKey) -> Option<Row>where
LockType: 'static,
Row: Archive + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<<Row as StorableRow>::WrappedRow as Archive>::Archived: Deserialize<<Row as StorableRow>::WrappedRow, HighDeserializer<Error>>,
pub fn select(&self, pk: PrimaryKey) -> Option<Row>where
LockType: 'static,
Row: Archive + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<<Row as StorableRow>::WrappedRow as Archive>::Archived: Deserialize<<Row as StorableRow>::WrappedRow, HighDeserializer<Error>>,
Selects Row from table identified with provided primary key. Returns None if no value presented.
pub fn insert(&self, row: Row) -> Result<PrimaryKey, WorkTableError>where
Row: Archive + Clone + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<Row as StorableRow>::WrappedRow: Archive + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<<Row as StorableRow>::WrappedRow as Archive>::Archived: ArchivedRowWrapper,
PrimaryKey: Clone,
AvailableTypes: 'static,
AvailableIndexes: AvailableIndex,
SecondaryIndexes: TableSecondaryIndex<Row, AvailableTypes, AvailableIndexes>,
LockType: 'static,
pub fn insert_cdc<SecondaryEvents>(
&self,
row: Row,
) -> Result<(PrimaryKey, Operation<<PkGen as PrimaryKeyGeneratorState>::State, PrimaryKey, SecondaryEvents>), WorkTableError>where
Row: Archive + Clone + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<Row as StorableRow>::WrappedRow: Archive + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<<Row as StorableRow>::WrappedRow as Archive>::Archived: ArchivedRowWrapper,
PrimaryKey: Clone,
SecondaryIndexes: TableSecondaryIndex<Row, AvailableTypes, AvailableIndexes> + TableSecondaryIndexCdc<Row, AvailableTypes, SecondaryEvents, AvailableIndexes>,
PkGen: PrimaryKeyGeneratorState,
AvailableIndexes: Debug + AvailableIndex,
Sourcepub async fn reinsert(
&self,
row_old: Row,
row_new: Row,
) -> Result<PrimaryKey, WorkTableError>where
Row: Archive + Clone + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<Row as StorableRow>::WrappedRow: Archive + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<<Row as StorableRow>::WrappedRow as Archive>::Archived: ArchivedRowWrapper,
PrimaryKey: Clone,
AvailableTypes: 'static,
AvailableIndexes: Debug + AvailableIndex,
SecondaryIndexes: TableSecondaryIndex<Row, AvailableTypes, AvailableIndexes>,
LockType: 'static,
pub async fn reinsert(
&self,
row_old: Row,
row_new: Row,
) -> Result<PrimaryKey, WorkTableError>where
Row: Archive + Clone + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<Row as StorableRow>::WrappedRow: Archive + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<<Row as StorableRow>::WrappedRow as Archive>::Archived: ArchivedRowWrapper,
PrimaryKey: Clone,
AvailableTypes: 'static,
AvailableIndexes: Debug + AvailableIndex,
SecondaryIndexes: TableSecondaryIndex<Row, AvailableTypes, AvailableIndexes>,
LockType: 'static,
Reinserts provided row with updating indexes and saving it’s data in new place. Is used to not delete and insert because this situation causes a possible gap when row doesn’t exist.
For reinsert it’s ok that part of indexes will lead to old row and other
part is for new row. Goal is to make PrimaryKey of the row always
acceptable. As for reinsert PrimaryKey will be same for both old and
new Link’s, goal will be achieved.
pub fn reinsert_cdc<SecondaryEvents>(
&self,
row_old: Row,
row_new: Row,
) -> Result<(PrimaryKey, Operation<<PkGen as PrimaryKeyGeneratorState>::State, PrimaryKey, SecondaryEvents>), WorkTableError>where
Row: Archive + Clone + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<Row as StorableRow>::WrappedRow: Archive + for<'a> Serialize<Strategy<Serializer<AlignedVec, ArenaHandle<'a>, Share>, Error>>,
<<Row as StorableRow>::WrappedRow as Archive>::Archived: ArchivedRowWrapper,
PrimaryKey: Clone,
SecondaryIndexes: TableSecondaryIndex<Row, AvailableTypes, AvailableIndexes> + TableSecondaryIndexCdc<Row, AvailableTypes, SecondaryEvents, AvailableIndexes>,
PkGen: PrimaryKeyGeneratorState,
AvailableIndexes: Debug + AvailableIndex,